#ifndef Automaton_H
#define Automaton_H

#include "Map.h"
#include "Topology.h"
#include "Neighborhood.h"
#include "TowerNeighborhood.h"

#include <vector>

///The Automaton class is the main class in the library, represents a Cellular Automata type. 
/**
 * This class uses a type of neighborhood and a type of topology in a map 
 * class, to create the final graph, this graph is a net of references to 
 * a matrix, the base element matrix. 
 * This class contain the next step function, the method to calculate the 
 * next inner states of the elements in the base element matrix, given by 
 * the graph form and the change state algorithm given by the method. 
*/ 
template<class T> class Automaton{
	
public:
    //--------------------start of the inner variables--------------------------
	///ToDo:
	std::vector<int> position;    

	///This structure represent a vertex in graph with all is properies.
	/**
	* The Node objects represents the node in the graph, 
	* contain a pointer to a state value in the values matrix,
	* and a pointers to other states in the matrix representing 
	* the edge structure
	*/
    typedef struct  Node{
      
      public:
          
      ///The pointer to the game board
      T* centerCell;
      ///The end index in the vector of Neighbor's pointer
      int endNeighbor;
      ///The begin index in the vector of Neighbor's pointer
      int beginNeighbor;
	  
	  ///ToDo:
	  Node(T* cell, int begin, int end):centerCell(cell), beginNeighbor(begin), endNeighbor(end){}
      
    };
    /********************************************************/

    ///the pointer to the grid states
    T *pointerToNodes;
    ///ToDo:
	T *copyPointerToNodes;
	///this is used in createNode method
	T *copyPointerToNodes2;
    
    /**The cointainer of all the pointer's ("edges") 
	representing the neighbor's */
	std::vector<T*> neighbors;
    
    ///the graph container (node structure container)
    std::vector<Node> graph;
    
    /**
	* this array's represents the dimensionality of the Automaton
	* (the size of the array's dimension's)the number of the i 
	* element in the array represent the number of "cell's" in the 
	* i dimension.
	*/
    static const int maxDimensions = 100;
    ///ToDo:
	int dimSizes[maxDimensions];
    ///ToDo:
	int copyDimSizes[maxDimensions];
	//-----------------------------------------------
    
	///ToDo:
	int dimension;
	////////////////////////////////////////////

	/**This int value is to "remember where starts, and finish"
	some neighborhood in the vertex's vector*/
    int indexHood;
    
    /**
	* the map is the objet who correlate the topology of the 
	* graph, and the creation of the neighborhood's(the vertex of 
	* the graphs)
	*/
    Map *map;
	///ToDo:
    Topology *topology;
    ///ToDo:
	Neighborhood *neighborhood; 
    
    ///The states variable is for combinatory
    int states;
    //--------------end of the inner variables-----------------------------
                
public:
	///The constructor 
    Automaton(T *gridPointer, int Dsizes[], int dimesions):pointerToNodes(gridPointer), dimension(dimesions){
    /*T *gridPointer() is the moving pointer over the
	grid, used like an iterator, in "ndimensional arrays",
	Dsizes[] is the pointer to the dimension sizes array,
	and int dimension is the number of dimensions of the 
	array */
			
		//set the index of neighbors to 0
		indexHood = 0;

		//create the dimension objects
		//one for each domension
		for(int i=0;i < dimension; ++i){
			dimSizes[i] = Dsizes[i];              
		}
		//then create a toppolgy objects and put in 
		//the dimension objects
        topology = new Topology(dimension);
        for(int i=0;i<dimension;++i){
			topology->setDimensionSize(i, Dsizes[i]);
		}
		//last create a neighborhood object and create
		//the map with the topology and neighborhood objects
        neighborhood = new TowerNeighborhood(); 
        map = new Map(topology, neighborhood);
	}
    
    /// The copy-constructor:
    Automaton(const Automaton& a){
		pointerToNodes = a.pointerToNodes; 
		dimension = a.dimension;
		topology = a.topology;
		dimSizes = a.dimSizes;
		neighborhood = a.neighborhood;
		map = a.map;
		//set the index of neighbors to 0
		indexHood = 0;

	}
    
	///Overwrite the = operator to avoid problems when you forget initialize whit the constructor  
    Automaton& operator=(const Automaton& a) {
		pointerToNodes = a.pointerToNodes; 
		dimension = a.dimension;
		topology = a.topology;
		dimSizes = a.dimSizes;
		neighborhood = a.neighborhood;
		map = a.map;
		//set the index of neighbors to 0
		indexHood = 0;
        return *this;
    }   
	///ToDo:
	void setMap();
    ///ToDo:
	void getMap();
    ///ToDo:
	void setNeighborhood();
    ///ToDo:
	void getNeighborhood();
    /**
    With this function you can iterate over an array independently
	of the dimension and element type of the array, this is done 
	tnx to the pointer to the first element of the array
	pointerToNodes, and the dimension sizes array dimSizes[maxDimensions].
     
	In this method you iterate over the states array to point
	all the elements independently of the dimension of the array
	and call the createNode() method, for each element in the array
	, heach element in the array are  "center cell", the "node" in 
	the graph, but in the array there is no asociation betwen the
	elements, so in createNode() method, the pointer to the 
	element is keep and the edge structure of the node is set and
	keep, in the structure, for each element is create a Node 
	structure, and the structures are push in a vector container,
	the graph, now the relationships betwen the edges and nodes 
	are set, and the iteration over the array is done in the one
	dimencional	vector container, the graph, tnx to the 
	structures elements.
	*/    
	void iterateOverTheSatesArray(int index=0);
	///ToDo:
	void nextStep();
	/**
	This method is for create the structure container of the
	couples node and edge/s,the "structure" container is the graph,
	the structure consist of a center pointer to the states matrix,
	the node,and two index for the begining and end of the 
	neighbor's vector container,the edge's(links to the 
	neighborhoods).
    
	This method use the copyDimSizes position array, this
	vector define the node coordinates in the grid, and the 
	setEdge(position) method from Map class is used to 
	find the edge, map is the mixin betwem the topology class 
	and the neighborhood class.
	The generated edge(pointers to elements in the states arrays) 
	are keep, in the neighbors vector container, and is size 
	determine the index for the node.
	*/
    void createNode();
	/**
	If you want to access any element in a multi-dimensional
    array, use a pointer to the first element and advance 
    the pointer to the position of the element that you want
    to change/access. 
	This method change the copy pointer(copyPointerToNodes) 
	global variable to point a element in the states array given by 
	the vector<int> position argument
	*/ 
    void getNodePointer(std::vector<int> position);
	/**
	If you want to access any element in a multi-dimensional
     array, pass a pointer to the first element and advance 
     the pointer to the position of the element that you want
     to change.
        
     This method change the copy pointer(copyP) to a element in 
    the states array given by the position argument
	*/ 
    void getNodePointer(int position[]);
    ///This method change the copy pointer(copyP) to a element in the states array given by the position argument
	/** 
	If you want to access any element in a multi-dimensional
    array, use a pointer to the first element and advance 
    the pointer to the position of the element that you want
    to change/access. 
	This method change the copy pointer(copyPointerToNodes) 
	global variable to point a element in the states array given by 
	the one dimenssional array, the position vector, the global 
	array copyDimSizes
	*/
	void getNodePointer();
    
};
template<class T>
void Automaton<T>::getNodePointer()
    {
    
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes; 
        /*whe need a int to count the positions 
		to move the pointer */
        int move = 0 ;
        
        //for all the elements in the given position
		//(the copyDimSizes position variable)
		//the copyDimSizes size is = dimension 
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension){
                //multiply all the position index's for 
                //each dimension size    
                move = copyDimSizes[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                  copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += copyDimSizes[i];
        } 
}//---------------getNodePointer()--------------------------

template<class T>   
void Automaton<T>::getNodePointer(std::vector<int> position)
    {
    
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes;
        /*whe need a int to count the positions 
		to move the pointer */
        int move = 0 ;
        
        //for all the elements in the given position
		//(vector<int> position argument)
		//the position size is = dimension 
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension ){
                //multiply all the position index's for 
                //each dimension size    
                move = position[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                //and move that number of times the pointer                                    
                copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += position[i];
        } 
    }//---------------getNodePointer(vector<int> position)--------------------------

template<class T>   
void Automaton<T>::getNodePointer(int position[])
    {
       
        //first point the copy pointer to the [0][0][0]...[0]
        //position, the begining element of the array
        copyPointerToNodes = pointerToNodes;
        //whe need a int to count the positions to move the
        //pointer 
        int move = 0 ;
        
        //for all the elements in the given positions
        for(int i=0; i<dimension; i++ ){
            //if is not the last element
            if( (i+1) < dimension ){
                //multiply all index of the position for the 
                //dimension of is respective dimension in the array     
                move = position[i];
                for(int j = (i+1); j < dimension; j++){        
                    move = (move)*(dimSizes[j]);
                }
                //and move that number of times the pointer                                    
                copyPointerToNodes += move;
            }
            //if is the last element move the pointer the position
            //number     
            else
                copyPointerToNodes += position[i];
        } 
    }//---------------getNodePointer(int position[])--------------------------   
template<class T>
void Automaton<T>::createNode(){
    
	//first get the pointer to the node "center cell"
	//copyPointerToNodes pointer
	getNodePointer();
	//because whe are goin to change copyPointerToNodes pointer
	//first keep the pointing direction
	copyPointerToNodes2 = copyPointerToNodes;
	
	
	//yo have to transform the position vector, the arraycopyDimSizes[]
	//to a vector container for use for the map
    position.clear();
    position.reserve(dimension);
    for(int i=0;i<dimension;++i){
        position.push_back(copyDimSizes[i]);
    }
	//calculate the neighborhood
    map->setEdge(position);
    std::vector< std::vector<int> > hood = map->getEdge();
	
	//next keep the edge (the pointer) in the neighbors container

	//for all the neighbors directions
    for(int i=0;i<int(hood.size());++i){
		//move the pointer to the element given by the direction
        getNodePointer(hood[i]);
		//and push the pointer in the container
        neighbors.push_back(copyPointerToNodes);    
    }
	//create the node
    Node cell(copyPointerToNodes2, indexHood, (indexHood + int(hood.size())-1));
    //and laste change the index array for the next time
	indexHood = indexHood + int(hood.size());
    
	//keep the structure in the grahp vector container
    graph.push_back(cell);
    
}//------------createNode()-----------------

template<class T>
void Automaton<T>::iterateOverTheSatesArray(int index){    
    
         //iterate over the first dimension 
        for( int i=0; i<dimSizes[index] ; ++i ){
            
			//now modify a copy of the dimension sizes array,
			//to use has the new position vector
			copyDimSizes[index] = i;                
                
                //if isn't the last dimension 
                if( (index + 1) < dimension){
                    /*use recursion to modify the copy of 
					the dimension sizes array, and start again
					but for the nexts dimensions
					*/
                    iterateOverTheSatesArray(index+1);                               
                }
                //if is the last dimension
                else{					
			/*modify a copy of the dimension sizes array,
			to use has the new position vector
            in the last dimension*/
					copyDimSizes[index] = i;
                    //and use the index (the new position vector)
                    //to create the Node and edge structure
                    createNode();
                }
        }       
}//----------iterateOverTheSatesArray-----------------

template<class T>
void Automaton<T>::nextStep(){
    
}//-----------nextStep()--------------------

#endif
