#ifndef Dimension_H
#define Dimension_H

///This type of objects represent one dimension
/**
* This type of objects represent one dimension
* (an axe in the "picture"), and can be normalized, 
* like a ring, or with bounds a strand. 
*/
class Dimension {

    ///the size or possible numbers of coordinates in this axe
    int maxLimit;
    ///is a ring (true) or a strand (with bounds)(false)
    bool normalized;
    
    public:
    
	/**
    * the constructor, creates a object, with a given 
    * size (100 default) and type of bounds (default with bounds)
	*/
    Dimension(int size=100, bool normalized=false): maxLimit(size), normalized(normalized){}
    
    /// The copy-constructor:
    Dimension(const Dimension& d): maxLimit(d.maxLimit), normalized(d.normalized){}
    
	/** Overwrite the = operator to avoid problems when you
    *forget initialize whit the constructor  
    */
	Dimension& operator=(const Dimension& d) {
        maxLimit = d.maxLimit;
        normalized = d.normalized;
        return *this;
    }   
    
	/// the destructor
    ~Dimension(){}    
                
    //-----------get set methods --------------------------
    ///Return the number of elements in this finite, discrete dimension object.
	inline int getSize(){return maxLimit;}
	///Set the number of elements in this finite, discrete dimension object.
    inline void setSize(int dsize){maxLimit = dsize;}
	/// Is this dimension a ring or a strand?.
	/**If this method return a true this dimension is a ring, false is a strand.*/
    inline bool getNormalized(){return normalized;}  
	///Set or unset the normalization in this dimension.
	/**If is set true the dimension is a ring if not (false) is a strand.*/  
    inline void setNormalized(bool normal){normalized = normal;}
    //-----------get set methods --------------------------
    
    /**
	* this method is to know if a coordinate
    * is outside of the bounds, returns true is the ordinate is 
	* out of the bounds false other way
	*/ 
    inline bool isOutOfBounds(int coordinate){    
    
		//If the dimension is normalized there is no bounds    
		if(normalized)
			return false;
		//if not normalized
		else{
		//and the coordinate  is to big or to small
		//is outside of bounds
			if( (0 <= coordinate)&&(coordinate < maxLimit) )
                return false;                               
			else
                return true;
		}
	}//---------isOutOfBounds(int coordinate)------------------
   
    /**
	* this method is to return the position in the dimension, in 
	* base of the coordinate, only has sense to call it if the dim
	* is normalized, but sometimes you want generic code and call 
	* it anyway so you have to call isOutOfBounds(int coordinate) 
	* method before call this method to avoid a bug
	*/
    inline int getPos(int coordinate){
		if(normalized){
			//if the coordinate is not in the bounds
			if(! ((0<= coordinate) && (coordinate < maxLimit)) ){
				if(0 > coordinate){
					/*if the coordinate is negative move
					one time the dimension to the left 
					and try again the function */
					coordinate = getPos(coordinate += maxLimit);
					return coordinate; 
				}                
				else{                
					/*if the coordinate is bigger than the
					limits move one time the dimension to 
					the right and try again the function*/
					coordinate = getPos(coordinate -= maxLimit);                       
					return coordinate;                     
				}
			}    
			else {//if the coordinate is in between boundary's
				return coordinate;
			}                
		}
		//not normalized
		else{ 
			return coordinate;
		}
	}//-------------------getPos(int coordinate)----------------      
};
#endif //Dimension_H
